/*====================================+=====================================+
! File CServerList.h                  ! Copyright (C) 2018-2019 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/listctrl.h>
#include <wx/timer.h>
#include "common/sc_base.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CServer
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CServer()
      {  ; }
      CServer( const wxString &s_ref ) : m_s_ref( s_ref )
      {  ; }
      //~CServer() ;

      /*--( Availability )-------------------------------------------------*/
      enum e_avail { AVA_UNKNOWN, AVA_OK, AVA_KO, AVA_NB } ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString m_s_name  ;
      wxString m_s_ref   ;
      wxString m_s_dc    ;
      wxString m_s_price ;
      wxString m_s_spec  ;
      /*-------------------------------------------------------------------*/
      bool     m_boo_select = false ;
      e_avail  m_avail      = AVA_UNKNOWN ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_vec_server = std::vector< CServer > ;

/*--( URLs associated to different regions )-------------------------------*/
struct str_region
{   /*---------------------------------------------------------------------*/
    wxString m_s_code ; // Region code : fr, de etc.
    wxString m_s_name ; // Region name
    const char *m_p_c_url_server ; // Main server page
    const char *m_p_c_url_order  ; // Server order page
    /*---------------------------------------------------------------------*/
} ;
/*-------------------------------------------------------------------------*/
using t_vec_region = std::vector< str_region > ;

/*-------------------------------------------------------------------------*/
class CServerList : public wxListCtrl
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CServerList( wxWindow *parent ) ;
      ~CServerList() ;
      /*-------------------------------------------------------------------*/
      enum e_col { COL_CHECK, COL_NAME, COL_REF, COL_DC, COL_PRICE, COL_SPEC,
                   COL_NB
                 } ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      t_vec_server m_vec_server ;
      /*-------------------------------------------------------------------*/
      const char *m_co_p_c_main_url = "https://www.kimsufi.com" ;
      t_vec_region m_vec_region ;

  /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxImageList *m_il   = nullptr ;
      int m_i_img_check   = -1 ;
      int m_i_img_uncheck = -1 ;
      /*-------------------------------------------------------------------*/
      sc::t_map_strint m_map_dc_ico ;
      /*-------------------------------------------------------------------*/
      int m_i_nb_refresh_server = 0 ;
      int m_i_nb_check_avail    = 0 ;
      /*-------------------------------------------------------------------*/
      wxTimer m_timer_check_avail ;
      bool    m_boo_check_batch_alert_done ;
      /*--( item attributes for available and not available servers )------*/
      wxItemAttr m_a_lia[ 2 ][ 2 ] ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxItemAttr *OnGetItemAttr( long l_item ) const ;
      // virtual wxItemAttr *OnGetItemColumnAttr( long item, long column
      //                                        ) const ;
      virtual int OnGetItemColumnImage( long l_item, long l_column ) const ;
      //virtual int OnGetItemImage( long item ) const ;
      virtual wxString OnGetItemText( long l_item, long l_column ) const ;
      void OnMouseLeftDown( wxMouseEvent &event ) ;
      void OnKeyDown( wxKeyEvent &event ) ;
      void add_checkboxes() ;
      /*-------------------------------------------------------------------*/
      long get_next_item_hil( long l_item_from ) const
      {  return GetNextItem( l_item_from,
                              wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED
                            ) ;
      }
      long get_item_focus() const
      {  return GetNextItem( -1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED ) ; }
      void refresh_all_items() ;
      /*-------------------------------------------------------------------*/
      void string_to_serv_ref_sel( const wxString &s_val ) ;
      wxString serv_ref_sel_to_string() const ;
      /*-------------------------------------------------------------------*/
      int  set_select( long l_item, bool boo_select ) ;
      void select_all( bool boo_select ) ;
      void select_invert() ;
      void select_unselect_hil_following_focus() ;
      const CServer *get_server( long l_item ) const ;
      CServer *get_server( long l_item ) ;
      /*-------------------------------------------------------------------*/
      int add_img_il( const wxBitmap &bmp ) ;
      void string_to_col_width( const wxString &s_val ) ;
      wxString col_width_to_string() const ;
      /*-------------------------------------------------------------------*/
      int launch_refresh_server() ;
      int refresh_server( int i_err_code, const wxString &co_s_data ) ;
      int launch_check_avail() ;
      int check_avail( int i_err_code,
                       const wxString &co_s_ref,
                       const wxString &co_s_data
                     ) ;
      /*-------------------------------------------------------------------*/
      bool get_auto_check_avail_running() const
      {  return m_timer_check_avail.IsRunning() ; }
      int start_auto_check_avail() ;
      int stop_auto_check_avail() ;
      /*-------------------------------------------------------------------*/
      wxString get_url( long l_item ) ;
      const str_region *get_region( const wxString &s_region ) const ;
      const str_region *get_region() const ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CServerList.h                          !
+==========================================================================*/
