/*====================================+=====================================+
! File CDlgPrefs.cpp                  ! Copyright (C) 2018-2019 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include "CFrame.h"
#include "CApp.h"
#include "CDlgPrefs.h"
#include "app_info.h"
#include "common/sc_version.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CPanPrefsGen::CPanPrefsGen( wxWindow *parent, CDlgPrefs *dlg )
            : CPanPrefs( parent, dlg )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*--( Lang )------------------------------------------------------------*/
   init_cho_lang() ;
   /*--( Region )----------------------------------------------------------*/
   init_cho_region() ;
   /*--( Update )----------------------------------------------------------*/
   sc::fill_cho_chkupd_freq( m_cho_chkupd_freq ) ;
   m_cho_chkupd_freq->SetSelection( wxGetApp().M_i_chkupd_freq.get() ) ;
   m_cho_chkupd_freq->Bind( wxEVT_CHOICE,
                            [ this ]( wxCommandEvent &event )
                            {  init_chkupd_next() ; }
                          ) ;
   init_chkupd_next() ;
   /*----------------------------------------------------------------------*/
   m_chk_startup_auto_check_avail->SetValue(
                              wxGetApp().M_boo_startup_auto_check_avail.get()
                                           ) ;
   m_chk_startup_systray_only->SetValue(
                                  wxGetApp().M_boo_startup_systray_only.get()
                                       ) ;
   /*----------------------------------------------------------------------*/
   m_i_check_avail_period->Bind( wxEVT_TEXT,
                                 [ this ]( wxCommandEvent &event )
                                 {  init_period_hms() ; }
                               ) ;
   m_i_check_avail_period->SetValue( wxGetApp().M_i_check_avail_period.get()
                                   ) ;
   /*--( The init has to be done manually during the UI creation )---------*/
   init_period_hms() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::build_ui()
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;

   /*--( Language )--------------------------------------------------------*/
   wxBoxSizer *bsz_lang = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_lang->Add( new wxStaticText( this, wxID_ANY, _( "Language" ) ),
                  0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                ) ;
   m_cho_lang = new wxChoice( this, wxID_ANY ) ;

   bsz_lang->Add( m_cho_lang, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_lang ) ;

   /*--( Update )----------------------------------------------------------*/
   wxBoxSizer *bsz_upd = new wxBoxSizer( wxHORIZONTAL ) ;

   bsz_upd->Add( new wxStaticText( this, wxID_ANY, _( "Check for update" ) ),
                 0, wxALL | wxALIGN_CENTER_VERTICAL, 5
               ) ;

   m_cho_chkupd_freq = new wxChoice( this, wxID_ANY ) ;
   bsz_upd->Add( m_cho_chkupd_freq, 0, wxALL, 5 ) ;

     m_stx_chkupd_next
   = new wxStaticText( this, wxID_ANY, wxEmptyString ) ;
   bsz_upd->Add( m_stx_chkupd_next, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5 ) ;
   bsz_gen->Add( bsz_upd ) ;

   /*--( Region )----------------------------------------------------------*/
   wxBoxSizer *bsz_reg = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_reg->Add( new wxStaticText( this, wxID_ANY,
                                   _( "Currency and VAT region" )
                                 ),
                 0, wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
   m_cho_region = new wxChoice( this, wxID_ANY ) ;
   bsz_reg->Add( m_cho_region, 0, wxALL, 5 ) ;
   bsz_gen->Add( bsz_reg ) ;

   /*--( Startup options )-------------------------------------------------*/
   wxStaticBoxSizer *stbsz_sta
              = new wxStaticBoxSizer( wxVERTICAL, this, _( "At startup" ) ) ;
   /*--( At startup run automatic availability check )---------------------*/
     m_chk_startup_auto_check_avail
   = new wxCheckBox( this, wxID_ANY, _( "Start auto check availability" ) ) ;
   stbsz_sta->Add( m_chk_startup_auto_check_avail, 0, wxALL, 5 ) ;

   /*--( At startup don't display the window, use only the systray )-------*/
     m_chk_startup_systray_only
   = new wxCheckBox( this, wxID_ANY,
                     _( "Appear only in the systray/notification zone" )
                   ) ;
   stbsz_sta->Add( m_chk_startup_systray_only, 0, wxALL, 5 ) ;
   bsz_gen->Add( stbsz_sta, 0, wxEXPAND ) ;

   /*--( Refresh availability period (in seconds) )------------------------*/
   wxBoxSizer *bsz_rap = new wxBoxSizer( wxHORIZONTAL ) ;
   bsz_rap->Add( new wxStaticText( this, wxID_ANY,
                            _( "Check servers availability every (seconds)" )
                                 ),
                 0, wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
     m_i_check_avail_period
   = new wxSpinCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                     wxSize( 80, -1 ), wxSP_ARROW_KEYS, 1, 999999
                   ) ;
   bsz_rap->Add( m_i_check_avail_period, 0,
                 wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
     m_stx_check_avail_period_hms
   = new wxStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition,
                       wxSize( 80, -1 )
                     ) ;
   bsz_rap->Add( m_stx_check_avail_period_hms, 0,
                 wxALIGN_CENTER_VERTICAL | wxALL, 5
               ) ;
   bsz_gen->Add( bsz_rap ) ;

   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_default()
{
   /*--( Lang : default is the first one )---------------------------------*/
   m_cho_lang->SetSelection( 0 ) ;
   /*--( Update : default is the first one )-------------------------------*/
   m_cho_chkupd_freq->SetSelection( wxGetApp().M_i_chkupd_freq.get_def() ) ;
   /*----------------------------------------------------------------------*/
   m_chk_startup_auto_check_avail->SetValue(
                          wxGetApp().M_boo_startup_auto_check_avail.get_def()
                                             ) ;
   m_chk_startup_systray_only->SetValue(
                              wxGetApp().M_boo_startup_systray_only.get_def()
                                       ) ;
   m_i_check_avail_period->SetValue(
                                  wxGetApp().M_i_check_avail_period.get_def()
                                   ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::apply_changes( wxUint32 &ui32_ppa )
{
   /*----------------------------------------------------------------------*/
   wxString s_lang = get_cho_lang_sel_canon() ;
   if( wxGetApp().M_s_lang.get() != s_lang )
   {  wxGetApp().M_s_lang.set( s_lang ) ; ui32_ppa |= PPA_RESTART ; }
   /*----------------------------------------------------------------------*/
   wxString s_region = get_cho_region_sel() ;
   if( wxGetApp().M_s_region.get() != s_region )
   {  wxGetApp().M_s_region.set( s_region ) ;
      ui32_ppa |= PPA_RELOAD_SERVER_LIST ;
   }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_i_chkupd_freq.set( m_cho_chkupd_freq->GetSelection() ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().M_boo_startup_auto_check_avail.set(
                                   m_chk_startup_auto_check_avail->GetValue()
                                                ) ;
   wxGetApp().M_boo_startup_systray_only.set(
                                       m_chk_startup_systray_only->GetValue()
                                            ) ;
   /*----------------------------------------------------------------------*/
   int i_check_avail_period = m_i_check_avail_period->GetValue() ;
   if( wxGetApp().M_i_check_avail_period.get() != i_check_avail_period )
   {  /*-------------------------------------------------------------------*/
      wxGetApp().M_i_check_avail_period.set( i_check_avail_period ) ;
      /*--( If active, the timer has to be restarted )---------------------*/
      if( wxGetApp().m_frame->m_server_list->get_auto_check_avail_running() )
      {  ui32_ppa |= PPA_RESTART_AUTO_REFRESH ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::append_cho_lang( const wxString &s_desc,
                                    const wxString &s_canon
                                  )
{  m_cho_lang->Append( s_desc, new wxStringClientData( s_canon ) ) ; }

/*-------------------------------------------------------------------------*/
wxString CPanPrefsGen::get_cho_lang_sel_canon()
{
   /*----------------------------------------------------------------------*/
   int  i_sel = m_cho_lang->GetCurrentSelection() ;
   void *p_object = m_cho_lang->GetClientObject( i_sel ) ;
   /*----------------------------------------------------------------------*/
   return p_object == nullptr
          ? wxString()
          : reinterpret_cast< wxStringClientData * >( p_object )->GetData() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_cho_lang()
{
   /*----------------------------------------------------------------------*/
   wxArrayString    as_canon       ;
   sc::t_map_strstr map_desc_canon ;
   size_t           sz_lang_sel    ;

   /*----------------------------------------------------------------------*/
     as_canon
   = wxTranslations::Get()->GetAvailableTranslations( APP_NAME ) ;
   /*--( Load the couples into a map to get it sorted on description )-----*/
   for( const auto &s_canon : as_canon )
   {
      /*-------------------------------------------------------------------*/
      const wxLanguageInfo *p_li = wxLocale::FindLanguageInfo( s_canon ) ;
      if( p_li != nullptr )
      {  map_desc_canon[ wxGetTranslation( p_li->Description ) ] = s_canon ;}
      /*-------------------------------------------------------------------*/
   }

   /*--( The default will be displayed first )-----------------------------*/
   sz_lang_sel = m_cho_lang->GetCount() ;
   m_cho_lang->Append( _( "System (if translation exists)" ) ) ;

   /*--( Then the couples sorted on description (sort done by "map") )-----*/
   for( const auto &desc_canon : map_desc_canon )
   {
      /*--( Language chosen by the user ? (non default one) )--------------*/
      if(    !wxGetApp().M_s_lang.empty()
          && wxGetApp().M_s_lang.get() == desc_canon.second
        )
      {  sz_lang_sel = m_cho_lang->GetCount() ; }
      /*-------------------------------------------------------------------*/
      append_cho_lang( desc_canon.first, desc_canon.second ) ;
      /*-------------------------------------------------------------------*/
   }

   /*-----------------------------------------------------------------------+
   ! Finally the "no translation". It is mainly added for test purpose.     !
   +-----------------------------------------------------------------------*/
   if(    !wxGetApp().M_s_lang.empty()
       && as_canon.Index( wxGetApp().M_s_lang.get() ) == wxNOT_FOUND
     )
   {  sz_lang_sel = m_cho_lang->GetCount() ; }
   /*----------------------------------------------------------------------*/
   append_cho_lang( _( "No translation (English)" ), "**none**" ) ;

   /*----------------------------------------------------------------------*/
   m_cho_lang->SetSelection( sz_lang_sel ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_chkupd_next()
{
   /*----------------------------------------------------------------------*/
   wxString s_date_next
          = sc::next_chkupd_iso( wxGetApp().M_s_chkupd_last.get(),
                                 m_cho_chkupd_freq->GetSelection()
                               ) ;
   if( !s_date_next.empty() )
   {  /*--( Convert from ISO to system default format )--------------------*/
      wxDateTime dt_next ;
      dt_next.ParseISODate( s_date_next ) ;
      s_date_next = dt_next.Format( "%x" ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_stx_chkupd_next->SetLabelText( s_date_next ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CPanPrefsGen::get_cho_region_sel()
{
   /*----------------------------------------------------------------------*/
   int  i_sel = m_cho_region->GetCurrentSelection() ;
   void *p_object = m_cho_region->GetClientObject( i_sel ) ;
   /*----------------------------------------------------------------------*/
   return p_object == nullptr
          ? wxString()
          : reinterpret_cast< wxStringClientData * >( p_object )->GetData() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::append_cho_region( const wxString &s_name,
                                      const wxString &s_region
                                    )
{  m_cho_region->Append( s_name, new wxStringClientData( s_region ) ) ; }

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_cho_region()
{
   /*----------------------------------------------------------------------*/
   CServerList *server_list = wxGetApp().m_frame->m_server_list ;
   /*----------------------------------------------------------------------*/
   wxString s_region = wxGetApp().M_s_region.get() ;
   size_t sz_region_sel = 0 ;
   /*--( Then the couples sorted on description (sort done by "map") )-----*/
   for( auto &region : server_list->m_vec_region )
   {
      /*-------------------------------------------------------------------*/
      append_cho_region( region.m_s_name, region.m_s_code ) ;
      /*--( Language chosen by the user ? (non default one) )--------------*/
      if( region.m_s_code == s_region )
      {  sz_region_sel = m_cho_region->GetCount() - 1 ; }
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   m_cho_region->SetSelection( sz_region_sel ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsGen::init_period_hms()
{
   /*----------------------------------------------------------------------*/
   m_stx_check_avail_period_hms->SetLabel(
                            sc::sec2hms( m_i_check_avail_period->GetValue() )
                                         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPanPrefsNet::CPanPrefsNet( wxWindow *parent, CDlgPrefs *dlg )
            : CPanPrefs( parent, dlg )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   m_txt_proxy_server->SetValue( wxGetApp().M_s_proxy_server.get() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsNet::build_ui()
{
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;

   /*----------------------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_proxy
              = new wxStaticBoxSizer( wxVERTICAL, this, _( "Proxy" ) ) ;
   wxFlexGridSizer *fgsz_proxy = new wxFlexGridSizer( 2, 0, 0 ) ;
   fgsz_proxy->AddGrowableCol( 1 ) ;
   fgsz_proxy->Add( new wxStaticText( this, wxID_ANY, _( "Server" ) ),
                    0, wxALIGN_CENTER_VERTICAL | wxALL, 5
                  ) ;
   m_txt_proxy_server = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                        wxDefaultPosition, wxDefaultSize, 0
                                      ) ;


   fgsz_proxy->Add( m_txt_proxy_server, 1, wxEXPAND | wxALL, 5 ) ;

   fgsz_proxy->AddStretchSpacer( 1 ) ;
   fgsz_proxy->Add( new wxStaticText( this, wxID_ANY,
                          _( "The server address must follow the format :\n"
                             "[scheme://][username:password@]server[:port]\n"
                             "For example :\n"
                             "socks://127.0.0.1:8080\n"
                             "http://kimfo:mypwd@myproxy.net:10000"
                           )
                                    ),
                    0, wxLEFT | wxRIGHT | wxBOTTOM, 5
                  ) ;

   stbsz_proxy->Add( fgsz_proxy, 0, wxALL | wxEXPAND, 5 ) ;

   bsz_gen->Add( stbsz_proxy, 0, wxALL | wxEXPAND, 5 ) ;

   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsNet::init_default()
{
   /*----------------------------------------------------------------------*/
   m_txt_proxy_server->SetValue( wxGetApp().M_s_proxy_server.get_def() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPanPrefsNet::apply_changes( wxUint32 &ui32_ppa )
{
   /*----------------------------------------------------------------------*/
   wxString s_proxy_server = m_txt_proxy_server->GetValue() ;
   if( wxGetApp().M_s_proxy_server.get() != s_proxy_server )
   {  wxGetApp().M_s_proxy_server.set( s_proxy_server ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgPrefs::CDlgPrefs()
         : wxPropertySheetDialog( this, wxID_ANY, wxEmptyString,
                                  wxDefaultPosition, wxDefaultSize,
                                  wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                                )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   sc::init_tlw_rect( this, wxGetApp().M_rec_prefs ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgPrefs::~CDlgPrefs()
{
   /*----------------------------------------------------------------------*/
   sc::save_tlw_rect( this, wxGetApp().M_rec_prefs ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgPrefs::build_ui()
{
   /*--( Set window title using standard value )---------------------------*/
   SetTitle(   wxString( APP_DISPLAY_NAME )
             + " - " + wxGetStockLabel( wxID_PREFERENCES, wxSTOCK_NOFLAGS )
           ) ;
   /*----------------------------------------------------------------------*/
   wxBookCtrlBase *book = GetBookCtrl() ;
   /*----------------------------------------------------------------------*/
   book->AddPage( new CPanPrefsGen( book, this ), _( "General" ) ) ;
   book->AddPage( new CPanPrefsNet( book, this ), _( "Network" ) ) ;
   /*--( Help button is used to restore data to defaults )-----------------*/
   CreateButtons( wxOK | wxAPPLY | wxCANCEL | wxHELP ) ;
   /*----------------------------------------------------------------------*/
   wxButton *bt_ok = static_cast< wxButton * >( FindWindow( wxID_OK ) ) ;
   if( bt_ok != nullptr )
   {  bt_ok->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtOk, this ) ; }
   /*----------------------------------------------------------------------*/
   wxButton *bt_apply = static_cast< wxButton * >( FindWindow( wxID_APPLY ));
   if( bt_apply != nullptr )
   {  bt_apply->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtApply, this ) ; }
   /*----------------------------------------------------------------------*/
   wxButton *bt_reinit_page
          = static_cast< wxButton * >( FindWindow( wxID_HELP ) ) ;
   if( bt_reinit_page != nullptr )
   {  bt_reinit_page->SetLabel( _( "Reset current page" ) ) ;
      bt_reinit_page->Bind( wxEVT_BUTTON, &CDlgPrefs::OnBtResetPage, this ) ;
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgPrefs::apply_changes()
{
   /*----------------------------------------------------------------------*/
   wxUint32 ui32_ppa = PPA_NOTHING ;
   /*--( Each panel will tell if a restart is needed ... )-----------------*/
   for( size_t sz_num = 0 ; sz_num < get_nb_page() ; ++sz_num )
   {
      /*-------------------------------------------------------------------*/
      get_page( sz_num )->apply_changes( ui32_ppa ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   CServerList *server_list = wxGetApp().m_frame->m_server_list ;
   /*----------------------------------------------------------------------*/
   if( ( ui32_ppa & PPA_RELOAD_SERVER_LIST ) != 0 )
   {  server_list->launch_refresh_server() ; }
   /*----------------------------------------------------------------------*/
   if( ( ui32_ppa & PPA_RESTART_AUTO_REFRESH ) != 0 )
   {
      /*-------------------------------------------------------------------*/
      server_list->stop_auto_check_avail() ;
      server_list->start_auto_check_avail() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   if( ( ui32_ppa & PPA_RESTART ) != 0 ) { m_boo_restart = true ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgPrefs::get_modal()
{
   /*----------------------------------------------------------------------*/
   ShowModal() ;
   /*----------------------------------------------------------------------*/
   if(    m_boo_restart
       && wxMessageBox( _( "Some changes need a restart to be active\n\n"
                           "Restart now ?"
                         ),
                         _( "Restart" ) + ' ' + APP_DISPLAY_NAME + " ?",
                         wxYES_NO
                      ) == wxYES
     )
   {  wxGetApp().restart_on_exit() ; }
   /*----------------------------------------------------------------------*/
   return GetReturnCode() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CDlgPrefs.cpp                          !
+==========================================================================*/
