/*====================================+=====================================+
! File CFrame.cpp                     ! Copyright (C) 2018-2019 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/wupdlock.h>
#include <wx/tokenzr.h>
#include "app_info.h"
#include "CFrame.h"
#include "CApp.h"
#include "common/sc_download.h"
#include "CDlgAbout.h"
#include "CDlgFirstContact.h"
#include "CDlgPrefs.h"
#include "common/sc_version.h"
#include "avail_16x16.xpm"
#include "avail_32x32.xpm"
#include "tango/preferences_desktop_16x16.xpm"
#include "tango/preferences_desktop_32x32.xpm"
#include "tango/network_server_16x16.xpm"
#include "tango/network_server_32x32.xpm"
#include "tango/media_playback_start_16x16.xpm"
#include "tango/media_playback_start_32x32.xpm"
#include "tango/media_playback_stop_16x16.xpm"
#include "tango/media_playback_stop_32x32.xpm"
#include "tango/software_update_available_16x16.xpm"
#include "tango/system_users_16x16.xpm"
/*-------------------------------------------------------------------------*/
#ifndef __WXMSW__
#include "app_64x64.xpm"
#endif // __WXMSW__
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CTaskBarIcon::CTaskBarIcon()
{
   /*----------------------------------------------------------------------*/
   CFrame *frame = wxGetApp().m_frame ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         [ frame ]( wxCommandEvent &event )
         {  wxPostEvent( frame, event ) ; }
       ) ;
   Bind( wxEVT_TASKBAR_LEFT_DCLICK,
         [ frame ]( wxTaskBarIconEvent &event )
         {  wxCommandEvent event_toggle_show( wxEVT_MENU, ID_TOGGLE_SHOW ) ;
            wxPostEvent( frame, event_toggle_show ) ;
         }
       ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenu *CTaskBarIcon::CreatePopupMenu()
{
   /*----------------------------------------------------------------------*/
   wxMenu    *menu    = new wxMenu()        ;
   CFrame    *frame   = wxGetApp().m_frame  ;
   wxMenuBar *menubar = frame->GetMenuBar() ;
   /*----------------------------------------------------------------------*/
   sc::append_menu( menu, menubar->GetMenu( 0 ) ) ;
   /*----------------------------------------------------------------------*/
   return menu ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CFrame::CFrame() : wxFrame( nullptr, wxID_ANY, wxEmptyString,
                            wxDefaultPosition, wxDefaultSize,
                            wxDEFAULT_FRAME_STYLE, "KIMFO"
                          )
{  ; }

/*-------------------------------------------------------------------------*/
CFrame::~CFrame()
{
   /*----------------------------------------------------------------------*/
   delete m_taskbar_icon ;
   /*----------------------------------------------------------------------*/
   sc::save_tlw_rect( this, wxGetApp().M_rec_main ) ;
   wxGetApp().M_i_spl_hor_pos.set( m_spl_hor->GetSashPosition() ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().M_s_serv_ref_sel.set( m_server_list->serv_ref_sel_to_string());
   wxGetApp().M_s_serv_list_col_width.set(
                                         m_server_list->col_width_to_string()
                                         ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().save_config_info() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnToggleAutoRefreshAvailable( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   if( m_server_list->get_auto_check_avail_running() )
   {  m_server_list->stop_auto_check_avail() ; }
   else
   {  m_server_list->start_auto_check_avail() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnPrefs( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   CDlgPrefs().get_modal() ;
   /*--( Restart needed and chosen by the user ? )-------------------------*/
   if( !wxGetApp().get_program_to_start_on_exit().empty() )
   {  Close( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::log_error( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_txt_info->SetDefaultStyle( wxTextAttr( *wxRED ) ) ;
   wxLogMessage( s ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::log_message( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_txt_info->SetDefaultStyle( wxTextAttr( wxNullColour ) ) ;
   wxLogMessage( s ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::log_message_ok( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_txt_info->SetDefaultStyle( wxTextAttr( *wxGREEN ) ) ;
   wxLogMessage( s ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_menubar()
{
   /*-----------------------------------------------------------------------+
   ! Menu                                                                   !
   +-----------------------------------------------------------------------*/
   wxMenuBar *menubar = new wxMenuBar ;
   wxMenu *mp ;

   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  force_show( !IsShown() ) ; },
         ID_TOGGLE_SHOW
       ) ;

   /*-----------------------------------------------------------------------+
   ! Menu File                                                              !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   mp->AppendCheckItem( ID_TOGGLE_SHOW, _( "&Show main window" ),
                        _( "Show or hide the main window" )
                      )->Check( true ) ;
   mp->AppendSeparator() ;
   sc::append_item( mp, ID_TOGGLE_AUTO_REFRESH_AVAIL, "_",
                    wxBitmap( media_playback_start_16x16_xpm )
                  ) ;
   sc::append_item( mp, ID_REFRESH_SERVER,
                    _( "Refresh &server list\tF5" ),
                    wxBitmap( network_server_16x16_xpm )
                  ) ;
   sc::append_item( mp, ID_CHECK_AVAIL,
                    _( "Check &availability\tF6" ),
                    wxBitmap( avail_16x16_xpm )
                  ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   sc::append_item( mp, wxID_PREFERENCES, wxEmptyString,
                    preferences_desktop_16x16_xpm
                  ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   sc::append_item( mp, wxID_EXIT, wxART_QUIT ) ;
   /*----------------------------------------------------------------------*/
   menubar->Append( mp, _( "&File" ) ) ;

   /*-----------------------------------------------------------------------+
   ! Menu Selection                                                         !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   sc::append_item( mp, ID_SEL_ALL, _( "&All\tCtrl+A" ),
                    wxNullBitmap, _( "Select all servers" )
                  ) ;
   sc::append_item( mp, ID_UNSEL_ALL, _( "&None\tCtrl+D" ),
                    wxNullBitmap, _( "Unselect all servers" )
                  ) ;
   sc::append_item( mp, ID_SEL_INV, _( "&Invert\tCtrl+I" ),
                    wxNullBitmap, _( "Invert current selection" )
                  ) ;
   sc::append_item( mp, ID_SEL_UNSEL_HIL,
                    _( "&Select/Unselect\tSpace" ), wxNullBitmap,
                    _( "Select/Unselect highlighted servers "
                       "following the current server"
                     )
                  ) ;
   menubar->Append( mp, _( "&Selection" ) ) ;

   /*-----------------------------------------------------------------------+
   ! Menu ?                                                                 !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   sc::append_item( mp, ID_MAKE_DONATION, _( "Make a &donation" ),
                    system_users_16x16_xpm
                  ) ;
   sc::append_item( mp, ID_FIRST_CONTACT, _( "&First contact" ),
                    sc::get_menu_bmp( wxART_INFORMATION )
                  ) ;
   sc::append_item( mp, ID_CHECK_VERSION_MANUAL, _( "&Check for update" ),
                    software_update_available_16x16_xpm
                  ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   sc::append_item( mp, wxID_ABOUT, wxART_INFORMATION ) ;
   menubar->Append( mp, "&?" ) ;
   SetMenuBar( menubar ) ;

   /*-----------------------------------------------------------------------+
   ! Menu options bindings                                                  !
   +-----------------------------------------------------------------------*/
   Bind( wxEVT_MENU, &CFrame::OnToggleAutoRefreshAvailable,
         this, ID_TOGGLE_AUTO_REFRESH_AVAIL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->launch_refresh_server() ; },
         ID_REFRESH_SERVER
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->launch_check_avail() ; },
         ID_CHECK_AVAIL
       ) ;
   Bind( wxEVT_MENU, &CFrame::OnPrefs, this, wxID_PREFERENCES ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { Close() ; },
         wxID_EXIT
       ) ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->select_all( true ) ; },
         ID_SEL_ALL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->select_all( false ) ; },
         ID_UNSEL_ALL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->select_invert() ; },
         ID_SEL_INV
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_server_list->select_unselect_hil_following_focus() ; },
         ID_SEL_UNSEL_HIL
       ) ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         []( wxCommandEvent &event )
         {  wxLaunchDefaultBrowser( APP_URL_DONATION ) ; },
         ID_MAKE_DONATION
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  /*--( Get version data )---------------------------------------*/
            sc::CDownload dl( this, APP_URL_VERSION ) ;
            int i_err = dl.non_threaded_exec() ;
            check_version( i_err, dl.get_def_data(), true ) ;
            /*-------------------------------------------------------------*/
         },
         ID_CHECK_VERSION_MANUAL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  CDlgFirstContact( this ).get_modal() ; },
         ID_FIRST_CONTACT
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  CDlgAbout( this ).get_modal() ; },
         wxID_ABOUT
       ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_toolbar()
{
   /*----------------------------------------------------------------------*/
   wxToolBar *toolbar = new wxToolBar( this, wxID_ANY ) ;
   wxMenuBar *menubar = GetMenuBar() ;

   /*----------------------------------------------------------------------*/
   toolbar->AddTool( ID_TOGGLE_AUTO_REFRESH_AVAIL, wxEmptyString,
                     wxBitmap( media_playback_start_32x32_xpm )
                   ) ;
   /*----------------------------------------------------------------------*/
   toolbar->AddSeparator() ;
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi = menubar->FindItem( ID_REFRESH_SERVER ) ;
   wxASSERT( mi != nullptr ) ;
   toolbar->AddTool( ID_REFRESH_SERVER, wxEmptyString,
                     wxBitmap( network_server_32x32_xpm ),
                     mi->GetItemLabelText()
                   ) ;
   /*----------------------------------------------------------------------*/
   mi = menubar->FindItem( ID_CHECK_AVAIL ) ;
   wxASSERT( mi != nullptr ) ;
   toolbar->AddTool( ID_CHECK_AVAIL, wxEmptyString,
                     wxBitmap( avail_32x32_xpm ),
                     mi->GetItemLabelText()
                   ) ;
   /*----------------------------------------------------------------------*/
   toolbar->AddSeparator() ;
   /*----------------------------------------------------------------------*/
   toolbar->AddTool( wxID_PREFERENCES, wxEmptyString,
                     wxBitmap( preferences_desktop_32x32_xpm ),
                     wxGetStockLabel( wxID_PREFERENCES, wxSTOCK_NOFLAGS )
                   ) ;
   /*----------------------------------------------------------------------*/
   toolbar->Realize() ;
   SetToolBar( toolbar ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init()
{
   /*----------------------------------------------------------------------*/
   const wxIcon co_ico(
                        #ifdef __WXMSW__
                           "A_APP_ICON"
                        #else
                           app_64x64_xpm
                        #endif // __WXMSW__
                      ) ;
   SetIcon( co_ico ) ;
   /*----------------------------------------------------------------------*/
   SetTitle( APP_DISPLAY_NAME " " APP_VERSION ) ;
   /*----------------------------------------------------------------------*/
   if( wxTaskBarIcon::IsAvailable() )
   {  m_taskbar_icon = new CTaskBarIcon() ;
      m_taskbar_icon->SetIcon( co_ico, GetTitle() ) ;
   }
   /*-----------------------------------------------------------------------+
   ! Build UI                                                               !
   +-----------------------------------------------------------------------*/
   m_panel = new wxPanel( this ) ;
   m_bsz = new wxBoxSizer( wxVERTICAL ) ;
   m_panel->SetSizer( m_bsz ) ;

   /*--( Splitter avoiding unsplit by setting the minimum size )-----------*/
   m_spl_hor = new wxSplitterWindow( m_panel, wxID_ANY, wxDefaultPosition,
                                     wxDefaultSize, wxSP_LIVE_UPDATE
                                   ) ;
   m_spl_hor->SetMinimumPaneSize( 1 ) ;
   m_bsz->Add( m_spl_hor, 1, wxALL | wxEXPAND, 5 ) ;

   /*--( Listing KS- "catalog" )-------------------------------------------*/
   m_server_list = new CServerList( m_spl_hor ) ;
   /*--( Select the servers that where selected at the end of last exec )--*/
   m_server_list->string_to_serv_ref_sel( wxGetApp().M_s_serv_ref_sel.get());

   /*--( Logs )------------------------------------------------------------*/
   m_txt_info = new wxTextCtrl( m_spl_hor, wxID_ANY, wxEmptyString,
                                wxDefaultPosition, wxDefaultSize,
                                  wxTE_MULTILINE | wxTE_READONLY
                                | wxHSCROLL | wxTE_RICH
                              ) ;
   sc::set_font_fix( m_txt_info, true ) ;
   wxLog::SetActiveTarget( new wxLogTextCtrl( m_txt_info ) ) ;

   /*--( Split )-----------------------------------------------------------*/
   m_spl_hor->SplitHorizontally( m_server_list, m_txt_info ) ;

   /*-----------------------------------------------------------------------+
   ! Position and size of the window                                        !
   +-----------------------------------------------------------------------*/
   if( wxGetApp().M_rec_main.get().GetSize() == wxDefaultSize )
   {
      /*-------------------------------------------------------------------*/
      wxRect rec_cur = wxGetApp().M_rec_main.get() ;
      const wxRect rec_scr = sc::get_screen_area_under( this ) ;
      /*--( Default frame size )-------------------------------------------*/
      rec_cur.width  = wxMax( 800, rec_scr.width  / 3.0 * 2.0 ) ;
      rec_cur.height = wxMax( 600, rec_scr.height / 4.0 * 3.0 ) ;
      /*-------------------------------------------------------------------*/
      rec_cur.SetPosition( wxDefaultPosition ) ;
      wxGetApp().M_rec_main.set( rec_cur ) ;
      /*-------------------------------------------------------------------*/
   }

   /*-----------------------------------------------------------------------+
   ! Download management                                                    !
   +-----------------------------------------------------------------------*/
   Bind( sc::EVT_DOWNLOAD,
         [ this ]( sc::CDownloadEvent &event )
         {  /*-------------------------------------------------------------*/
            switch( event.get_download_id() )
            {
               /*----------------------------------------------------------*/
               case DID_VERSION :
                  check_version( event.get_err_code(), event.get_string(),
                                 false
                               ) ;
                  break ;
               /*----------------------------------------------------------*/
               default :
                  wxFAIL_MSG( "Unmanaged download id" ) ;
                  break ;
               /*----------------------------------------------------------*/
            }
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      ) ;

   /*----------------------------------------------------------------------*/
   init_menubar() ;
   init_toolbar() ;
   gui_init_auto_check_avail() ;
   /*----------------------------------------------------------------------*/
   sc::init_tlw_rect( this, wxGetApp().M_rec_main ) ;
   /*----------------------------------------------------------------------*/
   int i_spl_hor = wxGetApp().M_i_spl_hor_pos.get() ;
   if( i_spl_hor < 0 )
   {  i_spl_hor = GetClientSize().GetHeight() / 4 * 3 ; }
   m_spl_hor->SetSashPosition( i_spl_hor ) ;

   /*----------------------------------------------------------------------*/
   if( !wxGetApp().M_boo_startup_systray_only.get() )
   {  Show( true ) ; }
   gui_init_toggle_show() ;

   /*----------------------------------------------------------------------*/
   if( wxGetApp().M_boo_startup_auto_check_avail.get() )
   {  m_server_list->start_auto_check_avail() ; }
   /*----------------------------------------------------------------------*/
   m_server_list->launch_refresh_server() ;

   /*--( Help the user the first time the application is started )---------*/
   if( wxGetApp().get_first_start_ever() )
   {  wxPostEvent( this, wxCommandEvent( wxEVT_MENU, ID_FIRST_CONTACT ) ) ; }

   /*----------------------------------------------------------------------*/
   if( sc::check_version_needed( wxGetApp().M_s_chkupd_last.get(),
                                 wxGetApp().M_i_chkupd_freq.get()
                               )
     )
   {  /*-------------------------------------------------------------------*/
      sc::CDownload dl( this, APP_URL_VERSION ) ;
      dl.get_dlbase().set_download_id( DID_VERSION ) ;
      dl.thread_start() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::gui_enable_refresh_server( bool boo_on )
{
   /*----------------------------------------------------------------------*/
   GetMenuBar()->Enable( ID_REFRESH_SERVER, boo_on ) ;
   GetToolBar()->EnableTool( ID_REFRESH_SERVER, boo_on ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::gui_enable_check_avail( bool boo_on )
{
   /*----------------------------------------------------------------------*/
   GetMenuBar()->Enable( ID_CHECK_AVAIL, boo_on ) ;
   GetToolBar()->EnableTool( ID_CHECK_AVAIL, boo_on ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::gui_init_auto_check_avail()
{
   /*----------------------------------------------------------------------*/
   wxMenuBar *menubar = GetMenuBar() ;
   wxToolBar *toolbar = GetToolBar() ;
   /*----------------------------------------------------------------------*/
   const bool co_boo_running = m_server_list->get_auto_check_avail_running();
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi = menubar->FindItem( ID_TOGGLE_AUTO_REFRESH_AVAIL ) ;
   mi->SetItemLabel( co_boo_running
                     ? _( "Sto&p automatic availability check\tF4" )
                     : _( "&Start automatic availability check\tF4" )
                   ) ;
   /*-----------------------------------------------------------------------+
   ! SetBitmap seems to only work properly on MSW. On other platforms like  !
   ! GTK it looks like the bitmap can't be changed when the item is part of !
   ! the menu (deleting/reinserting it can cause crashes)                   !
   +-----------------------------------------------------------------------*/
   mi->SetBitmap( co_boo_running
                  ? media_playback_stop_16x16_xpm
                  : media_playback_start_16x16_xpm
                ) ;

   toolbar->SetToolNormalBitmap( ID_TOGGLE_AUTO_REFRESH_AVAIL,
                                 co_boo_running
                                 ? media_playback_stop_32x32_xpm
                                 : media_playback_start_32x32_xpm
                               ) ;
   toolbar->SetToolShortHelp( ID_TOGGLE_AUTO_REFRESH_AVAIL,
                              mi->GetItemLabelText()
                            ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::gui_init_toggle_show()
{
   /*----------------------------------------------------------------------*/
   GetMenuBar()->FindItem( ID_TOGGLE_SHOW )->Check( IsShown() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::force_show( bool boo_show )
{
   /*----------------------------------------------------------------------*/
   Iconize( !boo_show ) ;
   Show( boo_show ) ;
   if( boo_show ) { Raise() ; }
   /*----------------------------------------------------------------------*/
   gui_init_toggle_show() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::alert_selected_server_avail()
{
   /*----------------------------------------------------------------------*/
   force_show( true ) ;
   RequestUserAttention( wxUSER_ATTENTION_INFO ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::check_version( int i_err_code, const wxString &s_data,
                            bool boo_check_manual
                          )
{
   /*-----------------------------------------------------------------------+
   ! Even if the last version data could not be retrieved the check is      !
   ! considered as done                                                     !
   +-----------------------------------------------------------------------*/
   wxGetApp().M_s_chkupd_last.set( wxDateTime::Today().FormatISODate() ) ;

   /*--( The two delimiters will handle CRLF & LF separators )-------------*/
   wxStringTokenizer tkz( s_data, "\r\n" ) ;
   wxString s_last_version ;
   wxString s_message ;
   /*----------------------------------------------------------------------*/
   if( tkz.HasMoreTokens() ) { s_last_version = tkz.GetNextToken() ; }

   /*--( Error ? )---------------------------------------------------------*/
   if(    i_err_code != sc::CDownloadBase::DE_NO_ERROR_DONE
       || s_last_version.empty()
     )
   {  /*-------------------------------------------------------------------*/
      if( boo_check_manual )
      {  s_message.Printf( _( "%s's last version data could not be "
                              "downloaded\n%s"
                            ),
                           APP_DISPLAY_NAME,
                           sc::CDownloadBase::st_get_err_msg( i_err_code )
                         ) ;
         wxMessageBox( s_message ) ;
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Same old ? )------------------------------------------------------*/
   if( APP_VERSION == s_last_version )
   {
      /*-------------------------------------------------------------------*/
      if( boo_check_manual )
      {  /*----------------------------------------------------------------*/
         s_message = _( "This version is the latest available" ) ;
         sc::CDlgLastVersion( this, s_message, s_data ).get_modal() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( New version ! )---------------------------------------------------*/
   s_message.Printf( _( "The version %s is available" ), s_last_version ) ;
   sc::CDlgNewVersion( this, s_message, s_data ).get_modal() ;
   /*--( After new version installed the program is restarted )------------*/
   if( !wxGetApp().get_program_to_start_on_exit().empty() )
   {  Close( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CFrame.cpp                           !
+==========================================================================*/
